@@ -1,6 +1,6 @@
-use proc_macro2::TokenStream;
+use proc_macro2::{Span, TokenStream};
 use syn;
-use syn::{Data, DeriveInput, Field, Fields};
+use syn::{Data, DeriveInput, Field, Fields, Ident};
 
 use util::*;
 
@@ -8,6 +8,7 @@ use util::*;
 struct ObjAttrs {
     name: Option<String>,
     description: Option<String>,
+    scalar: Option<Ident>,
 }
 
 impl ObjAttrs {
@@ -20,7 +21,9 @@ impl ObjAttrs {
         // Check attributes for name and description.
         if let Some(items) = get_graphql_attr(&input.attrs) {
             for item in items {
-                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "name", AttributeValidation::String)  {
+                if let Some(AttributeValue::String(val)) =
+                    keyed_item_value(&item, "name", AttributeValidation::String)
+                {
                     if is_valid_name(&*val) {
                         res.name = Some(val);
                         continue;
@@ -31,12 +34,20 @@ impl ObjAttrs {
                         );
                     }
                 }
-                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "description", AttributeValidation::String)  {
+                if let Some(AttributeValue::String(val)) =
+                    keyed_item_value(&item, "description", AttributeValidation::String)
+                {
                     res.description = Some(val);
                     continue;
                 }
+                if let Some(AttributeValue::String(scalar)) =
+                    keyed_item_value(&item, "scalar", AttributeValidation::String)
+                {
+                    res.scalar = Some(Ident::new(&scalar as &str, Span::call_site()));
+                    continue;
+                }
                 panic!(format!(
-                    "Unknown object attribute for #[derive(GraphQLObject)]: {:?}",
+                    "Unknown struct attribute for #[derive(GraphQLObject)]: {:?}",
                     item
                 ));
             }
@@ -49,7 +60,7 @@ impl ObjAttrs {
 struct ObjFieldAttrs {
     name: Option<String>,
     description: Option<String>,
-    deprecation: Option<String>,
+    deprecation: Option<DeprecationAttr>,
     skip: bool,
 }
 
@@ -60,10 +71,15 @@ impl ObjFieldAttrs {
         // Check doc comments for description.
         res.description = get_doc_comment(&variant.attrs);
 
+        // Check builtin deprecated attribute for deprecation.
+        res.deprecation = get_deprecated(&variant.attrs);
+
         // Check attributes.
         if let Some(items) = get_graphql_attr(&variant.attrs) {
             for item in items {
-                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "name", AttributeValidation::String)  {
+                if let Some(AttributeValue::String(val)) =
+                    keyed_item_value(&item, "name", AttributeValidation::String)
+                {
                     if is_valid_name(&*val) {
                         res.name = Some(val);
                         continue;
@@ -74,14 +90,29 @@ impl ObjFieldAttrs {
                         );
                     }
                 }
-                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "description", AttributeValidation::String)  {
+                if let Some(AttributeValue::String(val)) =
+                    keyed_item_value(&item, "description", AttributeValidation::String)
+                {
                     res.description = Some(val);
                     continue;
                 }
-                if let Some(AttributeValue::String(val)) = keyed_item_value(&item, "deprecation", AttributeValidation::String) {
-                    res.deprecation = Some(val);
+                if let Some(AttributeValue::String(val)) =
+                    keyed_item_value(&item, "deprecation", AttributeValidation::String)
+                {
+                    res.deprecation = Some(DeprecationAttr { reason: Some(val) });
                     continue;
                 }
+                match keyed_item_value(&item, "deprecated", AttributeValidation::String) {
+                    Some(AttributeValue::String(val)) => {
+                        res.deprecation = Some(DeprecationAttr { reason: Some(val) });
+                        continue;
+                    }
+                    Some(AttributeValue::Bare) => {
+                        res.deprecation = Some(DeprecationAttr { reason: None });
+                        continue;
+                    }
+                    None => {}
+                }
                 if let Some(_) = keyed_item_value(&item, "skip", AttributeValidation::Bare) {
                     res.skip = true;
                     continue;
@@ -150,7 +181,8 @@ pub fn impl_object(ast: &syn::DeriveInput) -> TokenStream {
         };
 
         let build_deprecation = match field_attrs.deprecation {
-            Some(s) => quote!{ field.deprecated(#s)  },
+            Some(DeprecationAttr { reason: Some(s) }) => quote!{ field.deprecated(Some(#s)) },
+            Some(DeprecationAttr { reason: None }) => quote!{ field.deprecated(None) },
             None => quote!{ field },
         };
 
@@ -170,8 +202,33 @@ pub fn impl_object(ast: &syn::DeriveInput) -> TokenStream {
         });
     }
 
+    let (_, ty_generics, _) = generics.split_for_impl();
+
+    let mut generics = generics.clone();
+
+    if attrs.scalar.is_none() {
+        generics.params.push(parse_quote!(__S));
+        {
+            let where_clause = generics.where_clause.get_or_insert(parse_quote!(where));
+            where_clause
+                .predicates
+                .push(parse_quote!(__S: juniper::ScalarValue));
+            where_clause
+                .predicates
+                .push(parse_quote!(for<'__b> &'__b __S: juniper::ScalarRefValue<'__b>));
+        }
+    }
+
+    let scalar = attrs
+        .scalar
+        .unwrap_or_else(|| Ident::new("__S", Span::call_site()));
+
+    let (impl_generics, _, where_clause) = generics.split_for_impl();
+
     let toks = quote! {
-        impl #generics ::juniper::GraphQLType for #ident #generics {
+        impl#impl_generics juniper::GraphQLType<#scalar> for #ident #ty_generics
+            #where_clause
+        {
             type Context = ();
             type TypeInfo = ();
 
@@ -185,8 +242,10 @@ pub fn impl_object(ast: &syn::DeriveInput) -> TokenStream {
 
             fn meta<'r>(
                 _: &(),
-                registry: &mut ::juniper::Registry<'r>
-            ) -> ::juniper::meta::MetaType<'r> {
+                registry: &mut juniper::Registry<'r, #scalar>
+            ) -> juniper::meta::MetaType<'r, #scalar>
+                where #scalar: 'r
+            {
                 let fields = &[
                     #(#meta_fields)*
                 ];
@@ -199,9 +258,9 @@ pub fn impl_object(ast: &syn::DeriveInput) -> TokenStream {
                 &self,
                 _: &(),
                 field_name: &str,
-                _: &::juniper::Arguments,
-                executor: &::juniper::Executor<Self::Context>
-            ) -> ::juniper::ExecutionResult
+                _: &juniper::Arguments<#scalar>,
+                executor: &juniper::Executor<Self::Context, #scalar>
+            ) -> juniper::ExecutionResult<#scalar>
             {
 
                 match field_name {
@@ -213,5 +272,22 @@ pub fn impl_object(ast: &syn::DeriveInput) -> TokenStream {
         }
     };
 
-    toks
+    let dummy_const = Ident::new(
+        format!("_IMPL_JUNIPER_SCALAR_VALUE_FOR_{}", ident).as_str(),
+        Span::call_site(),
+    );
+
+    quote!{
+        #[allow(non_upper_case_globals, unused_attributes, unused_qualifications)]
+        #[doc(hidden)]
+        const #dummy_const: () = {
+            mod juniper {
+                __juniper_use_everything!();
+            }
+
+            extern crate std;
+
+            #toks
+        };
+    }
 }
