@@ -10,6 +10,7 @@ extern crate proc_macro;
 extern crate proc_macro2;
 #[macro_use]
 extern crate quote;
+#[macro_use]
 extern crate syn;
 #[macro_use]
 extern crate lazy_static;
@@ -18,6 +19,7 @@ extern crate regex;
 mod derive_enum;
 mod derive_input_object;
 mod derive_object;
+mod derive_juniper_scalar_value;
 mod util;
 
 use proc_macro::TokenStream;
@@ -42,3 +44,10 @@ pub fn derive_object(input: TokenStream) -> TokenStream {
     let gen = derive_object::impl_object(&ast);
     gen.into()
 }
+
+#[proc_macro_derive(ScalarValue)]
+pub fn derive_juniper_scalar_value(input: TokenStream) -> TokenStream {
+    let ast = syn::parse::<syn::DeriveInput>(input).unwrap();
+    let gen = derive_juniper_scalar_value::impl_scalar_value(&ast);
+    gen.into()
+}
